/////////////////////////////////////////////////////
//pal2gba.cpp
//A quick and dirty program to convert microsoft .pal files
//generated by photoshop into an include file
/////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>

#define MAX_ENTRIES 256

typedef unsigned char  u8;
typedef unsigned short u16;
typedef unsigned long  u32;

typedef signed char  s8;
typedef signed short s16;
typedef signed long  s32;

////////////////////////////
//Define a structure for the garbage at the beginning of the file.
////////////////////////////
typedef struct tPalHeader{
	u8 garbage[22];
	u8 numEntries;
} PalHeader;

typedef struct tPalEntry {
	u8 a;
	u8 r;
	u8 g;
	u8 b;
} PalEntry;

void main (int argc, char *argv[]) {
	u32 start, end, fileSize, numEntries;
	s32 nameSize;
	PalHeader header;
	PalEntry entries[MAX_ENTRIES];
	FILE* inFile;
	FILE* outFile;
	char palName [256];
	char outFilename [256];

	//Check file size
	if (argc <= 1) {
		printf("\nUsage:  pal2gba file1.pal [file2.pal, file3.pal ...]\n");
		return;
	}

	strcpy(outFilename, "palettes.c");
	if (!(outFile = fopen(outFilename, "wt"))) {
			printf("Could not open %s for writing.", outFilename);
			return;
	}

	//Loop through all the files in the list, convert them over.
	for (int i = 1; i < argc; i++) {
		

		if (!(inFile = fopen(argv[i], "rb"))) {
			printf("Could not open %s for reading.", argv[i]);
			return;
		}

		fseek (inFile, 0, SEEK_SET);
		start = ftell (inFile);
		fseek (inFile, 0, SEEK_END);
		end = ftell (inFile);
		
		fileSize = (end - start)+1;
		fseek (inFile, 0, SEEK_SET);
		
		fread (&header, 1, sizeof(PalHeader), inFile);		
		fread (entries, fileSize, sizeof(PalEntry), inFile);	
		fclose(inFile);								//Close the file.

		numEntries = header.numEntries;
		
		
		nameSize = (s32)strchr(argv[i], '.');
		if (nameSize != NULL)
			nameSize = nameSize - (s32)argv[i];
		strncpy (palName, argv[i], nameSize);
		palName[nameSize] = '\0';
		


//		strcpy (outFilename, palName);
//		strcat(outFilename, ".pal.c");
		
		u16 color;

		fprintf(outFile, "const unsigned short %sPal[%d]={", palName, numEntries);
		for (u32 j = 0; j < numEntries; j++) {
			color = 0 | (entries[j].r >> 3) | ((entries[j].g >> 3) <<5) | ((entries[j].b >> 3) <<10);

			
			//fprintf(outFile, "\nr: %X, g: %X, b: %X", entries[j].r, entries[j].g, entries[j].b);

			fprintf(outFile, "\n%d,", color);
		}
		fprintf(outFile, "\n};\n\n");
	}
	
	fclose(outFile);	


}
